#include <iostream>
#include <fstream>
#include <vector>
#include <stdexcept>

#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/times.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pthread.h>
#include <errno.h>

using namespace std;



pid_t childpid; // pid of the process we're watching

// time before we send a SIGTERM
int delay;

// time in seconds between a SIGTERM and a SIGKILL sent to children
int delayBeforeKill=1;

const char *signalNames[]={
  "???",
  "SIGHUP",         
  "SIGINT",        
  "SIGQUIT",        
  "SIGILL",         
  "SIGTRAP",
  "SIGABRT",         
  "SIGBUS",          
  "SIGFPE",          
  "SIGKILL",         
  "SIGUSR1",         
  "SIGSEGV",         
  "SIGUSR2",         
  "SIGPIPE",         
  "SIGALRM",         
  "SIGTERM",         
  "SIGSTKFLT",       
  "SIGCHLD",         
  "SIGCONT",         
  "SIGSTOP",         
  "SIGTSTP",         
  "SIGTTIN",         
  "SIGTTOU",         
  "SIGURG",          
  "SIGXCPU",         
  "SIGXFSZ",         
  "SIGVTALRM",       
  "SIGPROF",         
  "SIGWINCH",        
  "SIGIO",           
  "SIGPWR",          
  "SIGSYS"};

const char *getSignalName(int sig)
{
  if (sig>0 && sig<=sizeof(signalNames)/sizeof(char *))
    return signalNames[sig];
  else
    return "???";
}

void sigHandler(int signum)
{
  static int what=0;

  switch(what)
  {
  case 0:
    cout << "\nSending SIGTERM to " << childpid << endl;
    kill(childpid,SIGTERM);
    cout << "Sleeping " << delayBeforeKill << " seconds" << endl;
    alarm(delayBeforeKill);
    ++what;
    break;
  case 1:
    cout << "Sending SIGKILL to " << childpid << endl;
    kill(childpid,SIGKILL);
    ++what;
    break;
  }
}

/**
 * run a solver
 */
void run(char **cmd)
{
  signal(SIGALRM,sigHandler);

  childpid=0;
    
  childpid=fork();
  if (childpid<0)
  {
    perror("fork failed");
    exit(127);
  }
  else
    if (childpid==0)
    {
      // child
      execvp(cmd[0],cmd);
      // only returns when it failed
      perror("exec failed");
      exit(127);
    }
    else
    {
      // parent
      alarm(delay);

      int childstatus;
      struct rusage childrusage;

      wait4(childpid,&childstatus,0,&childrusage);

      // the program we're watching (and all it subprocesses) have terminated
      cout << endl;

      if (WIFEXITED(childstatus))
	cout << "Child status: " << WEXITSTATUS(childstatus) << endl;
      else
	if (WIFSIGNALED(childstatus))
	{
	  int sig=WTERMSIG(childstatus);

	  cout << "Child ended because it received signal " 
	       << sig  << " (" << getSignalName(sig) << ")" << endl;
	}
	else
	  if (WIFSTOPPED(childstatus))
	  {
	    cout << "Child was stopped by signal "
		 << getSignalName(WSTOPSIG(childstatus))
		 << endl;
	  }
	  else
	  {
	    cout << "Child ended for unknown reason !!" << endl;
	  }

      if (WIFEXITED(childstatus))
	exit(WEXITSTATUS(childstatus));
    }
}

int main(int argc, char **argv)
{
  if (argc<3)
  {
    cout << "syntax: killer <delay> command" << endl;
    exit(1);
  }

  ofstream out("/tmp/watcherfile");

  try
  {
    ios_base::sync_with_stdio();

    cout.rdbuf(out.rdbuf());

    delay=atoi(argv[1]);

    run(&argv[2]);

    cout.flush();
  }
  catch (exception &e)
  {
    cout.flush();
    cerr << "\n\tUnexpected exception in killer:\n";
    cerr << "\t" << e.what() << endl;
    exit(1);
  }
}

// Local Variables:
// mode: C++
// End:

